@@ -83,7 +83,6 @@ enum {
 	ARG_BMAX_DIV,
 	ARG_DCV,
 	ARG_SEED,
-	ARG_CUTOFF,
 	ARG_PMAP,
 	ARG_NTOA,
 	ARG_USAGE,
@@ -103,7 +102,7 @@ static void printUsage(ostream& out) {
 	    << "    -C/--color              build a colorspace index" << endl
 	    << "    -a/--noauto             disable automatic -p/--bmax/--dcv memory-fitting" << endl
 	    << "    -p/--packed             use packed strings internally; slower, uses less mem" << endl
-	    << "    -B                      build both letter- and colorspace indexes" << endl
+	    //<< "    -B                      build both letter- and colorspace indexes" << endl
 	    << "    --bmax <int>            max bucket sz for blockwise suffix-array builder" << endl
 	    //<< "    --bmaxmultsqrt <int>    max bucket sz as multiple of sqrt(ref len)" << endl
 	    << "    --bmaxdivn <int>        max bucket sz as divisor of ref len (default: 4)" << endl
@@ -341,8 +340,7 @@ static void driver(const string& infile,
 				// Make sure the .3.ebwt and .4.ebwt files contain
 				// nucleotides; not colors
 				int numSeqs = 0;
-				std::pair<size_t, size_t> sztot2 =
-					fastaRefReadSizes(is, szs, plens, refparams, &bpout, numSeqs);
+				fastaRefReadSizes(is, szs, plens, refparams, &bpout, numSeqs);
 				refparams.color = true;
 				writeU32(fout3, szs.size(), bigEndian); // write # records
 				for(size_t i = 0; i < szs.size(); i++) {
@@ -355,7 +353,6 @@ static void driver(const string& infile,
 				int numSeqs2 = 0;
 				sztot = fastaRefReadSizes(is, szs, plens, refparams, NULL, numSeqs2);
 				assert_geq(numSeqs, numSeqs2);
-				//assert_eq(sztot2.second, sztot.second + numSeqs);
 			} else {
 				int numSeqs = 0;
 				sztot = fastaRefReadSizes(is, szs, plens, refparams, &bpout, numSeqs);
@@ -398,11 +395,9 @@ static void driver(const string& infile,
 				vector<RefRecord> szs2;
 				vector<uint32_t> plens2;
 				int numSeqs2 = 0;
-				std::pair<size_t, size_t> sztot2 =
-					fastaRefReadSizes(is, szs2, plens2, refparams, NULL, numSeqs2);
+				fastaRefReadSizes(is, szs2, plens2, refparams, NULL, numSeqs2);
 				assert_leq(numSeqs, numSeqs2);
 				// One less color than base
-				//assert_geq(sztot2.second, sztot.second + numSeqs);
 				refparams.color = true;
 			}
 #endif
