@@ -11,7 +11,6 @@
 #include <seqan/sequence.h>
 #include "alphabet.h"
 #include "assert_helpers.h"
-#include "spinlock.h"
 #include "threading.h"
 #include "bitset.h"
 #include "tokenize.h"
@@ -28,6 +27,7 @@
 
 using namespace std;
 using namespace seqan;
+using tthread::lock_guard;
 
 /// Constants for the various output modes
 enum output_types {
@@ -59,82 +59,6 @@ class Hit {
 public:
 	Hit() : stratum(-1) { }
 
-	/**
-	 * Initialize this Ent to represent the given hit.
-	 */
-	void toHitSetEnt(HitSetEnt& hse, AnnotationMap *amap) const {
-		hse.h = h;
-		hse.fw = fw ? 1 : 0;
-		hse.oms = oms;
-		hse.stratum = stratum;
-		hse.cost = cost;
-		if(mms.count() == 0) return;
-		for(int i = 0; i < (int)length(); i++) {
-			if(mms.test(i)) {
-				hse.expand();
-				hse.back().type = EDIT_TYPE_MM;
-				hse.back().pos = i;
-				hse.back().chr = refcs[i];
-			}
-		}
-	}
-
-	/**
-	 * Convert a list of Hit objects to a single HitSet object suitable
-	 * for chaining.
-	 */
-	static void toHitSet(const std::vector<Hit>& hits, HitSet& hs,
-	                     AnnotationMap *amap)
-	{
-		if(hits.empty()) return;
-		// Initialize HitSet
-		hs.name = hits.front().patName;
-		hs.seq  = hits.front().patSeq;
-		hs.qual = hits.front().quals;
-		hs.color = hits.front().color;
-		if(!hits.front().fw) {
-			// Re-reverse
-			reverseComplementInPlace(hs.seq, hs.color);
-			reverseInPlace(hs.qual);
-		}
-		// Convert hits to entries
-		hs.ents.resize(hits.size());
-		for(size_t i = 0; i < hs.ents.size(); i++) {
-			hits[i].toHitSetEnt(hs.ents[i], amap);
-		}
-	}
-
-	/**
-	 * Populate a vector of Hits with hits from the given HitSet.
-	 */
-	static void fromHitSet(std::vector<Hit>& hits, const HitSet& hs) {
-		assert(hs.sorted());
-		hits.resize(hs.size());
-		for(size_t i = 0; i < hs.size(); i++) {
-			hits[i].h = hs[i].h;
-			hits[i].oms = hs[i].oms;
-			hits[i].fw = hs[i].fw;
-			hits[i].stratum = hs[i].stratum;
-			hits[i].cost = hs[i].cost;
-			hits[i].mate = 0; //hs[i].mate;
-			hits[i].patName = hs.name;
-			hits[i].patSeq = hs.seq;
-			hits[i].quals = hs.qual;
-			hits[i].color = hs.color;
-			size_t len = seqan::length(hs.seq);
-			if(!hs[i].fw) {
-				reverseComplementInPlace(hits[i].patSeq, hs.color);
-				reverseInPlace(hits[i].quals);
-			}
-			hits[i].refcs.resize(len);
-			for(size_t j = 0; j < hs[i].size(); j++) {
-				if(hs[i][j].type != EDIT_TYPE_MM) continue;
-				hits[i].mms.set(hs[i][j].pos);
-				hits[i].refcs[hs[i][j].pos] = hs[i][j].chr;
-			}
-		}
-	}
-
 	U32Pair             h;       /// reference index & offset
 	U32Pair             mh;      /// reference index & offset for mate
 	uint32_t            patId;   /// read index
@@ -157,6 +81,8 @@ public:
 	                             ///            1 = upstream mate
 	                             ///            2 = downstream mate
 	bool                color;   /// read is in colorspace?
+	char                primer;  /// primer base, for csfasta files
+	char                trimc;   /// trimmed color, for csfasta files
 	uint32_t            seed;    /// pseudo-random seed for aligned read
 
 	/**
@@ -396,9 +322,9 @@ public:
 		ssmode_(ios_base::out)
 	{
 		_outs.push_back(out);
-		_locks.resize(1);
-		MUTEX_INIT(_locks[0]);
-		MUTEX_INIT(_mainlock);
+                vector<MUTEX_T*>::iterator it;
+                
+                _locks.push_back(new MUTEX_T);
 		initDumps();
 	}
 
@@ -428,10 +354,8 @@ public:
 		// Open all files for writing and initialize all locks
 		for(size_t i = 0; i < numOuts; i++) {
 			_outs.push_back(NULL); // we open output streams lazily
-			_locks.resize(i+1);
-			MUTEX_INIT(_locks[i]);
+			_locks.push_back(new MUTEX_T);
 		}
-		MUTEX_INIT(_mainlock);
 		initDumps();
 	}
 
@@ -458,7 +382,9 @@ public:
 	 * lock or any of the per-stream locks will be contended.
 	 */
 	void addWrapper() {
-		_numWrappers++;
+            numWrapper_mutex_m.lock();
+            _numWrappers++;
+            numWrapper_mutex_m.unlock();
 	}
 
 	/**
@@ -512,13 +438,12 @@ public:
 			out(h.h.first).writeChars(buf, ss.tellp());
 		}
 		unlock(hs[end-1].h.first);
-		mainlock();
+                lock_guard<MUTEX_T> guard(main_mutex_m);
 		commitHits(hs);
 		first_ = false;
 		numAligned_++;
 		if(paired) numReportedPaired_ += (end-start);
 		else       numReported_ += (end-start);
-		mainunlock();
 	}
 
 	void commitHit(const Hit& hit) {
@@ -631,7 +556,7 @@ public:
 	 * for example, outputting a read to an unaligned-read file.
 	 */
 	void mainlock() {
-		MUTEX_LOCK(_mainlock);
+                main_mutex_m.lock();
 	}
 
 	/**
@@ -639,7 +564,7 @@ public:
 	 * when, for example, outputting a read to an unaligned-read file.
 	 */
 	void mainunlock() {
-		MUTEX_UNLOCK(_mainlock);
+                main_mutex_m.unlock();
 	}
 
 	/**
@@ -685,7 +610,7 @@ public:
 		if(!p.paired() || onePairFile_) {
 			// Dump unpaired read to an aligned-read file of the same format
 			if(!dumpAlBase_.empty()) {
-				MUTEX_LOCK(dumpAlignLock_);
+                                lock_guard<MUTEX_T> guard(dumpAlignLock_);
 				if(dumpAl_ == NULL) {
 					assert(dumpAlQv_ == NULL);
 					dumpAl_ = openOf(dumpAlBase_, 0, "");
@@ -699,13 +624,12 @@ public:
 				if(dumpAlQv_ != NULL) {
 					dumpAlQv_->write(p.bufa().qualOrigBuf, p.bufa().qualOrigBufLen);
 				}
-				MUTEX_UNLOCK(dumpAlignLock_);
 			}
 		} else {
 			// Dump paired-end read to an aligned-read file (or pair of
 			// files) of the same format
 			if(!dumpAlBase_.empty()) {
-				MUTEX_LOCK(dumpAlignLockPE_);
+                                lock_guard<MUTEX_T> guard(dumpAlignLockPE_);
 				if(dumpAl_1_ == NULL) {
 					assert(dumpAlQv_1_ == NULL);
 					assert(dumpAlQv_2_ == NULL);
@@ -726,7 +650,6 @@ public:
 					dumpAlQv_1_->write(p.bufa().qualOrigBuf, p.bufa().qualOrigBufLen);
 					dumpAlQv_2_->write(p.bufb().qualOrigBuf, p.bufb().qualOrigBufLen);
 				}
-				MUTEX_UNLOCK(dumpAlignLockPE_);
 			}
 		}
 	}
@@ -741,7 +664,7 @@ public:
 		if(!p.paired() || onePairFile_) {
 			// Dump unpaired read to an unaligned-read file of the same format
 			if(!dumpUnalBase_.empty()) {
-				MUTEX_LOCK(dumpUnalLock_);
+                                lock_guard<MUTEX_T> guard(dumpUnalLock_);
 				if(dumpUnal_ == NULL) {
 					assert(dumpUnalQv_ == NULL);
 					dumpUnal_ = openOf(dumpUnalBase_, 0, "");
@@ -755,13 +678,12 @@ public:
 				if(dumpUnalQv_ != NULL) {
 					dumpUnalQv_->write(p.bufa().qualOrigBuf, p.bufa().qualOrigBufLen);
 				}
-				MUTEX_UNLOCK(dumpUnalLock_);
 			}
 		} else {
 			// Dump paired-end read to an unaligned-read file (or pair
 			// of files) of the same format
 			if(!dumpUnalBase_.empty()) {
-				MUTEX_LOCK(dumpUnalLockPE_);
+                                lock_guard<MUTEX_T> guard(dumpUnalLockPE_);
 				if(dumpUnal_1_ == NULL) {
 					assert(dumpUnal_1_ == NULL);
 					assert(dumpUnal_2_ == NULL);
@@ -780,7 +702,6 @@ public:
 					dumpUnalQv_1_->write(p.bufa().qualOrigBuf, p.bufa().qualOrigBufLen);
 					dumpUnalQv_2_->write(p.bufb().qualOrigBuf, p.bufb().qualOrigBufLen);
 				}
-				MUTEX_UNLOCK(dumpUnalLockPE_);
 			}
 		}
 	}
@@ -798,7 +719,7 @@ public:
 		if(!p.paired() || onePairFile_) {
 			// Dump unpaired read to an maxed-out-read file of the same format
 			if(!dumpMaxBase_.empty()) {
-				MUTEX_LOCK(dumpMaxLock_);
+                                lock_guard<MUTEX_T> guard(dumpMaxLock_);
 				if(dumpMax_ == NULL) {
 					dumpMax_ = openOf(dumpMaxBase_, 0, "");
 					assert(dumpMax_ != NULL);
@@ -810,13 +731,12 @@ public:
 				if(dumpMaxQv_ != NULL) {
 					dumpMaxQv_->write(p.bufa().qualOrigBuf, p.bufa().qualOrigBufLen);
 				}
-				MUTEX_UNLOCK(dumpMaxLock_);
 			}
 		} else {
 			// Dump paired-end read to a maxed-out-read file (or pair
 			// of files) of the same format
 			if(!dumpMaxBase_.empty()) {
-				MUTEX_LOCK(dumpMaxLockPE_);
+                                lock_guard<MUTEX_T> guard(dumpMaxLockPE_);
 				if(dumpMax_1_ == NULL) {
 					assert(dumpMaxQv_1_ == NULL);
 					assert(dumpMaxQv_2_ == NULL);
@@ -835,7 +755,6 @@ public:
 					dumpMaxQv_1_->write(p.bufa().qualOrigBuf, p.bufa().qualOrigBufLen);
 					dumpMaxQv_2_->write(p.bufb().qualOrigBuf, p.bufb().qualOrigBufLen);
 				}
-				MUTEX_UNLOCK(dumpMaxLockPE_);
 			}
 		}
 	}
@@ -845,9 +764,8 @@ public:
 	 * want to print a placeholder when output is chained.
 	 */
 	virtual void reportMaxed(vector<Hit>& hs, PatternSourcePerThread& p) {
-		mainlock();
+		lock_guard<MUTEX_T> guard(main_mutex_m);
 		numMaxed_++;
-		mainunlock();
 	}
 
 	/**
@@ -855,9 +773,8 @@ public:
 	 * want to print a placeholder when output is chained.
 	 */
 	virtual void reportUnaligned(PatternSourcePerThread& p) {
-		mainlock();
+                lock_guard<MUTEX_T> guard(main_mutex_m);
 		numUnaligned_++;
-		mainunlock();
 	}
 
 protected:
@@ -865,13 +782,12 @@ protected:
 	/// Implementation of hit-report
 	virtual void reportHit(const Hit& h) {
 		assert(h.repOk());
-		mainlock();
+                lock_guard<MUTEX_T> guard(main_mutex_m);
 		commitHit(h);
 		first_ = false;
 		if(h.mate > 0) numReportedPaired_++;
 		else           numReported_++;
 		numAligned_++;
-		mainunlock();
 	}
 
 	/**
@@ -894,7 +810,7 @@ protected:
 	 */
 	void lock(size_t refIdx) {
 		size_t strIdx = refIdxToStreamIdx(refIdx);
-		MUTEX_LOCK(_locks[strIdx]);
+		_locks[strIdx]->lock();
 	}
 
 	/**
@@ -905,7 +821,7 @@ protected:
 	 */
 	void unlock(size_t refIdx) {
 		size_t strIdx = refIdxToStreamIdx(refIdx);
-		MUTEX_UNLOCK(_locks[strIdx]);
+		_locks[strIdx]->unlock();
 	}
 
 	vector<OutFileBuf*> _outs;        /// the alignment output stream(s)
@@ -913,8 +829,9 @@ protected:
 	RecalTable         *recalTable_;  /// recalibration table
 	vector<string>*     _refnames;    /// map from reference indexes to names
 	int                 _numWrappers; /// # threads owning a wrapper for this HitSink
-	vector<MUTEX_T>     _locks;       /// pthreads mutexes for per-file critical sections
-	MUTEX_T             _mainlock;    /// pthreads mutexes for fields of this object
+	vector<MUTEX_T*>     _locks;       /// pthreads mutexes for per-file critical sections
+	MUTEX_T             main_mutex_m;    /// pthreads mutexes for fields of this object
+        MUTEX_T             numWrapper_mutex_m;
 
 	// Output filenames for dumping
 	std::string dumpAlBase_;
@@ -996,12 +913,6 @@ protected:
 		dumpAlignFlag_   = !dumpAlBase_.empty();
 		dumpUnalignFlag_ = !dumpUnalBase_.empty();
 		dumpMaxedFlag_   = !dumpMaxBase_.empty();
-		MUTEX_INIT(dumpAlignLock_);
-		MUTEX_INIT(dumpAlignLockPE_);
-		MUTEX_INIT(dumpUnalLock_);
-		MUTEX_INIT(dumpUnalLockPE_);
-		MUTEX_INIT(dumpMaxLock_);
-		MUTEX_INIT(dumpMaxLockPE_);
 	}
 
 	void destroyDumps() {
@@ -1486,262 +1397,6 @@ private:
 	uint32_t max_;
 };
 
-/**
- *
- */
-class ChainingHitSinkPerThread : public HitSinkPerThread {
-public:
-
-	ChainingHitSinkPerThread(HitSink& sink,
-	                         uint32_t n,
-	                         uint32_t max,
-	                         bool strata,
-	                         uint32_t mult) :
-	HitSinkPerThread(sink, max, n),
-	mult_(mult), strata_(strata), cutoff_(0xffff)
-	{
-		hs_ = NULL;
-		hsISz_ = 0;
-	}
-
-	/**
-	 * Return true iff we're allowed to span strata.
-	 */
-	virtual bool spanStrata() { return !strata_; }
-
-	/**
-	 * true -> we report best hits
-	 */
-	virtual bool best() { return true; }
-
-	/**
-	 * Report and then return false if we've already reported N.
-	 */
-	virtual bool reportHit(const Hit& h, int stratum) {
-		HitSinkPerThread::reportHit(h, stratum);
-		assert(hs_->sorted());
-		assert(hs_ != NULL);
-		assert_eq(h.stratum, stratum);
-		assert_eq(1, mult_);
-		assert(consistentStrata());
-		assert(!irrelevantCost(h.cost));
-
-		if(!hs_->empty() && strata_ && stratum < hs_->front().stratum) {
-			hs_->clear();
-			_bufferedHits.clear();
-			hitsForThisRead_ = 0;
-		}
-		assert(consistentStrata());
-
-		size_t replPos = 0;
-		if(!hs_->empty() && hs_->tryReplacing(h.h, h.fw, h.cost, replPos)) {
-			if(replPos != 0xffffffff) {
-				// Replaced an existing hit
-				assert_lt(replPos, _bufferedHits.size());
-				_bufferedHits[replPos] = h;
-				hs_->sort();
-			}
-			// Size didn't change, so no need to check against _max and _n
-			assert(hs_->sorted());
-			assert(consistentStrata());
-		} else {
-			// Added a new hit
-			hs_->expand();
-			hs_->back().h = h.h;
-			hs_->back().fw = h.fw;
-			hs_->back().stratum = h.stratum;
-			hs_->back().cost = h.cost;
-			hitsForThisRead_++;
-			if(hs_->size() > _max) {
-				assert_eq(hs_->size(), _bufferedHits.size());
-				return true; // done - report nothing
-			}
-			_bufferedHits.push_back(h);
-			if(hsISz_ == 0 &&
-			   hs_->size() == _n &&
-			   (_max == 0xffffffff || _max < _n))
-			{
-				assert_eq(hs_->size(), _bufferedHits.size());
-				return true; // already reported N good hits; stop!
-			}
-			hs_->sort();
-			assert(consistentStrata());
-		}
-		assert_eq(hs_->size(), _bufferedHits.size());
-		updateCutoff();
-		return false; // not at N yet; keep going
-	}
-
-	/**
-	 * Finalize current read by reporting any buffered hits from best
-	 * to worst until they're all reported or until we've reported all
-	 * N
-	 */
-	virtual uint32_t finishReadImpl() {
-		assert_eq(1, mult_);
-		assert(hs_ != NULL);
-		assert(consistentStrata());
-		uint32_t ret = hitsForThisRead_;
-		hitsForThisRead_ = 0;
-		if(!hs_->empty() && hs_->size() < _n) {
-			const size_t sz = _bufferedHits.size();
-			for(size_t i = 0; i < sz; i++) {
-				// Set 'oms' according to the number of other alignments
-				// at this stratum
-				_bufferedHits[i].oms = (sz / mult_) - 1;
-			}
-		}
-		std::sort(_bufferedHits.begin(), _bufferedHits.end(), HitCostCompare());
-		if(hs_->size() > _n) {
-			_bufferedHits.resize(_n);
-		}
-		assert(consistentStrata());
-		// Make sure that a chained, maxed-out read gets treated as
-		// maxed-out and not as unaligned
-		if(hs_->empty() && hs_->maxedStratum != -1) {
-			assert(_bufferedHits.empty());
-			// Boy, this is stupid.  Need to switch to just using HitSet
-			// internally all the time.
-			_bufferedHits.resize(_max+1);
-			for(size_t i = 0; i < _max+1; i++) {
-				_bufferedHits[i].stratum = hs_->maxedStratum;
-			}
-			ret = _max+1;
-		} else if(!hs_->empty() && hs_->maxedStratum != -1) {
-			assert_lt(hs_->front().stratum, hs_->maxedStratum);
-		}
-		return ret;
-	}
-
-	/**
-	 * Set the initial set of Hits.
-	 */
-	virtual bool setHits(HitSet& hs) {
-		hs_ = &hs;
-		assert(hs_ != NULL);
-		hsISz_ = hs.size();
-		cutoff_ = 0xffff;
-		hitsForThisRead_ = hs.size();
-		assert(_bufferedHits.empty());
-		assert_geq(hs.maxedStratum, -1);
-		assert_lt(hs.maxedStratum, 4);
-		if(!hs.empty()) {
-			assert_eq(-1, hs.maxedStratum);
-			hs.sort();
-			Hit::fromHitSet(_bufferedHits, hs);
-			assert(!_bufferedHits.empty());
-			assert_leq(_bufferedHits.size(), _max);
-			assert(consistentStrata());
-		} else if(hs.maxedStratum != -1) {
-			if(hs.maxedStratum == 0) {
-				cutoff_ = 0;
-				// Already done
-				return true;
-			}
-			cutoff_ = (hs.maxedStratum << 14);
-		}
-		assert_eq(hs_->size(), _bufferedHits.size());
-		updateCutoff();
-		return false;
-	}
-
-	/**
-	 * If we had any alignments at all and we're now moving on to a new
-	 * stratum, then we're done.
-	 */
-	virtual bool finishedWithStratumImpl(int stratum) {
-		assert(false);
-		return false;
-	}
-
-	/**
-	 * If there have been any hits reported so far, classify any
-	 * subsequent alignments with higher strata as irrelevant.
-	 */
-	virtual bool irrelevantCost(uint16_t cost) {
-		if(cutoff_ == 0) return false;
-		return cost > cutoff_;
-	}
-
-protected:
-
-#ifndef NDEBUG
-	/**
-	 * Sanity check that, if we're in strata_ mode, all 'stratum's
-	 * should be the same among hits.
-	 */
-	bool consistentStrata() {
-		if(hs_->empty() || !strata_) return true;
-		int stratum = hs_->front().stratum;
-		for(size_t i = 1; i < hs_->size(); i++) {
-			assert_eq(stratum, (*hs_)[i].stratum);
-		}
-		return true;
-	}
-#endif
-
-	/**
-	 * Update the lowest relevant cost.
-	 */
-	void updateCutoff() {
-		ASSERT_ONLY(uint16_t origCutoff = cutoff_);
-		assert(hs_->sorted());
-		assert(hs_->empty() || hs_->back().cost >= hs_->front().cost);
-		bool atCapacity = (hs_->size() >= _n);
-		if(atCapacity && (_max == 0xffffffff || _max < _n)) {
-			cutoff_ = min(hs_->back().cost, cutoff_);
-		}
-		if(strata_ && !hs_->empty()) {
-			uint16_t sc = hs_->back().cost;
-			sc = ((sc >> 14) + 1) << 14;
-			cutoff_ = min(cutoff_, sc);
-			assert_leq(cutoff_, origCutoff);
-		}
-	}
-
-	HitSet *hs_;
-	size_t hsISz_;
-	uint32_t mult_;
-	bool strata_; /// true -> reporting is stratified
-	uint16_t cutoff_; /// the smallest irrelevant cost
-};
-
-/**
- * Concrete factory for ChainingHitSinkPerThread.
- */
-class ChainingHitSinkPerThreadFactory : public HitSinkPerThreadFactory {
-public:
-	ChainingHitSinkPerThreadFactory(
-			HitSink& sink,
-			uint32_t n,
-			uint32_t max,
-			bool strata) :
-			sink_(sink),
-			n_(n),
-			max_(max),
-			strata_(strata)
-	{ }
-
-	/**
-	 * Allocate a new NGoodHitSinkPerThread object on the heap,
-	 * using the parameters given in the constructor.
-	 */
-	virtual HitSinkPerThread* create() const {
-		return new ChainingHitSinkPerThread(sink_, n_, max_, strata_, 1);
-	}
-	virtual HitSinkPerThread* createMult(uint32_t m) const {
-		uint32_t max = max_ * (max_ == 0xffffffff ? 1 : m);
-		uint32_t n = n_ * (n_ == 0xffffffff ? 1 : m);
-		return new ChainingHitSinkPerThread(sink_, n, max, strata_, m);
-	}
-
-private:
-	HitSink& sink_;
-	uint32_t n_;
-	uint32_t max_;
-	bool strata_;
-};
-
 /**
  * Report all valid alignments.
  */
@@ -2038,50 +1693,6 @@ private:
 	AnnotationMap *amap_;  ///
 };
 
-/**
- * Sink for outputting alignments in a binary format.
- */
-class ChainingHitSink : public HitSink {
-public:
-
-	/**
-	 * Construct a single-stream BinaryHitSink (default)
-	 */
-	ChainingHitSink(OutFileBuf* out, bool strata, AnnotationMap *amap, DECL_HIT_DUMPS2) :
-	HitSink(out, PASS_HIT_DUMPS2), amap_(amap), strata_(strata)
-	{
-		ssmode_ |= ios_base::binary;
-	}
-
-	/**
-	 * Report a batch of hits.
-	 */
-	virtual void reportHits(vector<Hit>& hs);
-
-	/**
-	 * Append a binary alignment to the output stream corresponding to
-	 * the reference sequence involved.
-	 */
-	virtual void append(ostream& o, const Hit& h) {
-		cerr << "Error: ChainingHitSink::append() not implemented" << endl;
-		throw 1;
-	}
-
-	/**
-	 * See hit.cpp
-	 */
-	virtual void reportMaxed(vector<Hit>& hs, PatternSourcePerThread& p);
-
-	/**
-	 * See hit.cpp
-	 */
-	virtual void reportUnaligned(PatternSourcePerThread& p);
-
-protected:
-	AnnotationMap *amap_;
-	bool strata_;
-};
-
 /**
  * Sink that does nothing.
  */
