@@ -14,7 +14,6 @@
 #include "assert_helpers.h"
 #include "tokenize.h"
 #include "random_source.h"
-#include "spinlock.h"
 #include "threading.h"
 #include "filebuf.h"
 #include "qual.h"
@@ -318,17 +317,6 @@ struct ReadBuf {
 		}
 	}
 
-	/**
-	 * Write read details to a HitSet object.
-	 */
-	void toHitSet(HitSet& hs) {
-		assert(!empty());
-		hs.name = name;
-		hs.seq = patFw;
-		hs.qual = qual;
-		hs.color = color;
-	}
-
 	static const int BUF_SIZE = 1024;
 
 	String<Dna5>  patFw;               // forward-strand sequence
@@ -394,7 +382,6 @@ class PatternSource {
 public:
 	PatternSource(uint32_t seed,
 	              bool randomizeQuals = false,
-	              bool useSpinlock = true,
 	              const char *dumpfile = NULL,
 	              bool verbose = false) :
 		seed_(seed),
@@ -402,9 +389,8 @@ public:
 		dumpfile_(dumpfile),
 		numWrappers_(0),
 		doLocking_(true),
-		useSpinlock_(useSpinlock),
 		randomizeQuals_(randomizeQuals),
-		lock_(),
+		mutex_m(),
 		verbose_(verbose)
 	{
 		// Open dumpfile, if specified
@@ -415,7 +401,6 @@ public:
 				throw 1;
 			}
 		}
-		MUTEX_INIT(lock_);
 	}
 
 	virtual ~PatternSource() { }
@@ -426,7 +411,9 @@ public:
 	 * whether locks will be contended.
 	 */
 	void addWrapper() {
-		numWrappers_++;
+            lock();
+            numWrappers_++;
+            unlock();
 	}
 
 	/**
@@ -535,17 +522,7 @@ public:
 	 */
 	void lock() {
 		if(!doLocking_) return; // no contention
-#ifdef USE_SPINLOCK
-		if(useSpinlock_) {
-			// User can ask to use the normal pthreads lock even if
-			// spinlocks are compiled in.
-			spinlock_.Enter();
-		} else {
-#endif
-			MUTEX_LOCK(lock_);
-#ifdef USE_SPINLOCK
-		}
-#endif
+		mutex_m.lock();
 	}
 
 	/**
@@ -554,17 +531,7 @@ public:
 	 */
 	void unlock() {
 		if(!doLocking_) return; // no contention
-#ifdef USE_SPINLOCK
-		if(useSpinlock_) {
-			// User can ask to use the normal pthreads lock even if
-			// spinlocks are compiled in.
-			spinlock_.Leave();
-		} else {
-#endif
-			MUTEX_UNLOCK(lock_);
-#ifdef USE_SPINLOCK
-		}
-#endif
+		mutex_m.unlock();
 	}
 
 	/**
@@ -633,15 +600,8 @@ protected:
 	ofstream out_;         /// output stream for dumpfile
 	int numWrappers_;      /// # threads that own a wrapper for this PatternSource
 	bool doLocking_;       /// override whether to lock (true = don't override)
-	/// User can ask to use the normal pthreads-style lock even if
-	/// spinlocks is enabled and compiled in.  This is sometimes better
-	/// if we expect bad I/O latency on some reads.
-	bool useSpinlock_;
 	bool randomizeQuals_;  /// true -> mess up qualities in a random way
-#ifdef USE_SPINLOCK
-	SpinLock spinlock_;
-#endif
-	MUTEX_T lock_; /// mutex for locking critical regions
+	MUTEX_T mutex_m; /// mutex for locking critical regions
 	bool verbose_;
 };
 
@@ -652,7 +612,6 @@ protected:
 class PairedPatternSource {
 public:
 	PairedPatternSource(uint32_t seed) {
-		MUTEX_INIT(lock_);
 		seed_ = seed;
 	}
 	virtual ~PairedPatternSource() { }
@@ -667,30 +626,19 @@ public:
 	 * fields is being updated.
 	 */
 	void lock() {
-#ifdef USE_SPINLOCK
-		spinlock_.Enter();
-#else
-		MUTEX_LOCK(lock_);
-#endif
+                mutex_m.lock();
 	}
 
 	/**
 	 * Unlock this PairedPatternSource.
 	 */
 	void unlock() {
-#ifdef USE_SPINLOCK
-		spinlock_.Leave();
-#else
-		MUTEX_UNLOCK(lock_);
-#endif
+		mutex_m.unlock();
 	}
 
 protected:
 
-#ifdef USE_SPINLOCK
-	SpinLock spinlock_;
-#endif
-	MUTEX_T lock_; /// mutex for locking critical regions
+	MUTEX_T mutex_m; /// mutex for locking critical regions
 	uint32_t seed_;
 };
 
@@ -848,7 +796,9 @@ public:
 	 * pair; returns false if ra contains a new unpaired read.
 	 */
 	virtual bool nextReadPair(ReadBuf& ra, ReadBuf& rb, uint32_t& patid) {
-		uint32_t cur = cur_;
+            lock();
+            uint32_t cur = cur_;
+            unlock();
 		while(cur < srca_.size()) {
 			if(srcb_[cur] == NULL) {
 				// Patterns from srca_[cur_] are unpaired
@@ -1089,12 +1039,11 @@ class TrimmingPatternSource : public PatternSource {
 public:
 	TrimmingPatternSource(uint32_t seed,
 	                      bool randomizeQuals = false,
-	                      bool useSpinlock = true,
 	                      const char *dumpfile = NULL,
 	                      bool verbose = false,
 	                      int trim3 = 0,
 	                      int trim5 = 0) :
-		PatternSource(seed, randomizeQuals, useSpinlock, dumpfile, verbose),
+		PatternSource(seed, randomizeQuals, dumpfile, verbose),
 		trim3_(trim3), trim5_(trim5) { }
 protected:
 	int trim3_;
@@ -1111,10 +1060,9 @@ public:
 	RandomPatternSource(uint32_t seed,
 	                    uint32_t numReads = 2000000,
 	                    int length = 35,
-	                    bool useSpinlock = true,
 	                    const char *dumpfile = NULL,
 	                    bool verbose = false) :
-		PatternSource(seed, false, useSpinlock, dumpfile, verbose),
+		PatternSource(seed, false, dumpfile, verbose),
 		numReads_(numReads),
 		length_(length),
 		seed_(seed)
@@ -1360,13 +1308,12 @@ public:
 	                    const vector<string>& v,
 	                    bool color,
 	                    bool randomizeQuals = false,
-	                    bool useSpinlock = true,
 	                    const char *dumpfile = NULL,
 	                    bool verbose = false,
 	                    int trim3 = 0,
 	                    int trim5 = 0,
 		                uint32_t skip = 0) :
-		TrimmingPatternSource(seed, randomizeQuals, useSpinlock,
+		TrimmingPatternSource(seed, randomizeQuals,
 		                      dumpfile, verbose, trim3, trim5),
 		color_(color), cur_(skip), skip_(skip), paired_(false), v_(),
 		quals_()
@@ -1546,13 +1493,12 @@ public:
 	                          const vector<string>& infiles,
 	                          const vector<string>* qinfiles,
 	                          bool randomizeQuals = false,
-	                          bool useSpinlock = true,
 	                          const char *dumpfile = NULL,
 	                          bool verbose = false,
 	                          int trim3 = 0,
 	                          int trim5 = 0,
 	                          uint32_t skip = 0) :
-		TrimmingPatternSource(seed, randomizeQuals, useSpinlock,
+		TrimmingPatternSource(seed, randomizeQuals,
 		                      dumpfile, verbose, trim3, trim5),
 		infiles_(infiles),
 		filecur_(0),
@@ -1772,7 +1718,6 @@ public:
 	                   const vector<string>* qinfiles,
 	                   bool color,
 	                   bool randomizeQuals,
-	                   bool useSpinlock = true,
 	                   const char *dumpfile = NULL,
 	                   bool verbose = false,
 	                   int trim3 = 0,
@@ -1782,7 +1727,7 @@ public:
 	                   bool intQuals = false,
 	                   uint32_t skip = 0) :
 		BufferedFilePatternSource(seed, infiles, qinfiles, randomizeQuals,
-		                          useSpinlock, dumpfile, verbose, trim3,
+		                          dumpfile, verbose, trim3,
 		                          trim5, skip),
 		first_(true), color_(color), solexa64_(solexa64),
 		phred64_(phred64), intQuals_(intQuals)
@@ -2022,7 +1967,6 @@ public:
 	                    const vector<string>& infiles,
 	                    bool color,
 	                    bool randomizeQuals = false,
-	                    bool useSpinlock = true,
 	                    const char *dumpfile = NULL,
 	                    bool verbose = false,
 	                    int trim3 = 0,
@@ -2032,7 +1976,7 @@ public:
 	                    bool intQuals = false,
 	                    uint32_t skip = 0) :
 		BufferedFilePatternSource(seed, infiles, NULL, randomizeQuals,
-		                          useSpinlock, dumpfile, verbose,
+		                          dumpfile, verbose,
 		                          trim3, trim5, skip),
 		color_(color),
 		solQuals_(solQuals),
@@ -2378,11 +2322,10 @@ public:
 			const vector<string>& infiles,
 			size_t length,
 			size_t freq,
-			bool useSpinlock = true,
 			const char *dumpfile = NULL,
 			bool verbose = false,
 			uint32_t skip = 0) :
-		BufferedFilePatternSource(seed, infiles, NULL, false, useSpinlock,
+		BufferedFilePatternSource(seed, infiles, NULL, false,
 		                          dumpfile, verbose, 0, 0, skip),
 		length_(length), freq_(freq),
 		eat_(length_-1), beginning_(true),
@@ -2520,7 +2463,6 @@ public:
 	                   const vector<string>& infiles,
 	                   bool color,
 	                   bool randomizeQuals = false,
-	                   bool useSpinlock = true,
 	                   const char *dumpfile = NULL,
 	                   bool verbose = false,
 	                   int trim3 = 0,
@@ -2531,7 +2473,7 @@ public:
 	                   bool fuzzy = false,
 	                   uint32_t skip = 0) :
 		BufferedFilePatternSource(seed, infiles, NULL, randomizeQuals,
-		                          useSpinlock, dumpfile, verbose,
+		                          dumpfile, verbose,
 		                          trim3, trim5, skip),
 		first_(true),
 		solQuals_(solexa_quals),
@@ -2931,13 +2873,12 @@ public:
 	                 const vector<string>& infiles,
 	                 bool color,
 	                 bool randomizeQuals = false,
-	                 bool useSpinlock = true,
 	                 const char *dumpfile = NULL,
 	                 bool verbose = false,
 	                 int trim3 = 0,
 	                 int trim5 = 0,
 	                 uint32_t skip = 0) :
-		BufferedFilePatternSource(seed, infiles, NULL, randomizeQuals, useSpinlock,
+		BufferedFilePatternSource(seed, infiles, NULL, randomizeQuals, 
 		                          dumpfile, verbose, trim3, trim5, skip),
 		first_(true), color_(color)
 	{ }
@@ -3076,12 +3017,11 @@ class ChainPatternSource : public BufferedFilePatternSource {
 public:
 	ChainPatternSource(uint32_t seed,
 	                   const vector<string>& infiles,
-	                   bool useSpinlock,
 	                   const char *dumpfile,
 	                   bool verbose,
 	                   uint32_t skip) :
 	BufferedFilePatternSource(
-		seed, infiles, NULL, false, useSpinlock, dumpfile, verbose, 0, 0, skip) { }
+		seed, infiles, NULL, false, dumpfile, verbose, 0, 0, skip) { }
 
 protected:
 
