@@ -1,9 +1,7 @@
 #!/bin/sh
 
 #
-# Downloads sequence for A. thaliana from TAIR.  This script was used
-# to build the Bowtie index for A. thaliana.  The downloaded version
-# was TAIR9.
+# Downloads sequence for A. thaliana from TAIR and builds Bowtie index.
 #
 
 GENOMES_MIRROR=ftp://ftp.arabidopsis.org/home/tair
@@ -33,19 +31,23 @@ if [ ! -x "$BOWTIE_BUILD_EXE" ] ; then
 	fi
 fi
 
+FC=
 for c in 1 2 3 4 5 C M ; do
-	if [ ! -f chr$c.fas ] ; then
-		F=${GENOMES_MIRROR}/Sequences/whole_chromosomes/chr$c.fas
+	if [ ! -f TAIR10_chr$c.fas ] ; then
+		FN=TAIR10_chr$c.fas
+		F=${GENOMES_MIRROR}/Sequences/whole_chromosomes/${FN}
+		[ -n "$FC" ] && FC="$FC,$FN"
+		[ -z "$FC" ] && FC=$FN
 		get $F || (echo "Error getting $F" && exit 1)
 	fi
 	
-	if [ ! -f chr$c.fas ] ; then
+	if [ ! -f TAIR10_chr$c.fas ] ; then
 		echo "Could not find chr$c.fas file!"
 		exit 2
 	fi
 done
 
-CMD="${BOWTIE_BUILD_EXE} $* chr1.fas,chr2.fas,chr3.fas,chr4.fas,chr5.fas,chrC.fas,chrM.fas  a_thaliana"
+CMD="${BOWTIE_BUILD_EXE} $* $FC a_thaliana"
 echo $CMD
 if $CMD ; then
 	echo "a_thaliana index built; you may remove fasta files"
