@@ -18,6 +18,8 @@ extern bool colorQual;
 extern int  snpPhred;
 extern bool showSeed;
 extern bool quiet;
+extern bool gAllowMateContainment;
+extern bool gReportColorPrimer;
 
 extern MUTEX_T gLock;
 
